@@ -16,71 +16,19 @@
  */
 package kafka.admin
 
-import java.util
 import java.util.Properties
 
 import kafka.admin.ConfigCommand.ConfigCommandOptions
 import kafka.common.InvalidConfigException
-import kafka.server.ConfigEntityName
-import kafka.utils.{Exit, Logging}
-import kafka.zk.{AdminZkClient, KafkaZkClient, ZooKeeperTestHarness}
-import org.apache.kafka.clients.admin._
-import org.apache.kafka.common.config.ConfigResource
-import org.apache.kafka.common.internals.KafkaFutureImpl
-import org.apache.kafka.common.Node
-import org.apache.kafka.common.security.scram.ScramCredentialUtils
-import org.apache.kafka.common.utils.Sanitizer
+import kafka.server.{ConfigEntityName, QuotaId}
+import kafka.utils.{Logging, ZkUtils}
+import kafka.zk.ZooKeeperTestHarness
+
 import org.easymock.EasyMock
 import org.junit.Assert._
 import org.junit.Test
 
-import scala.collection.mutable
-import scala.collection.JavaConverters._
-
 class ConfigCommandTest extends ZooKeeperTestHarness with Logging {
-
-  @Test
-  def shouldExitWithNonZeroStatusOnArgError(): Unit = {
-    assertNonZeroStatusExit(Array("--blah"))
-  }
-
-  @Test
-  def shouldExitWithNonZeroStatusOnZkCommandError(): Unit = {
-    assertNonZeroStatusExit(Array(
-      "--zookeeper", zkConnect,
-      "--entity-name", "1",
-      "--entity-type", "brokers",
-      "--alter",
-      "--add-config", "message.max.size=100000"))
-  }
-
-  @Test
-  def shouldExitWithNonZeroStatusOnBrokerCommandError(): Unit = {
-    assertNonZeroStatusExit(Array(
-      "--bootstrap-server", "invalid host",
-      "--entity-type", "brokers",
-      "--entity-name", "1",
-      "--describe"))
-  }
-
-  private def assertNonZeroStatusExit(args: Array[String]): Unit = {
-    var exitStatus: Option[Int] = None
-    Exit.setExitProcedure { (status, _) =>
-      exitStatus = Some(status)
-      throw new RuntimeException
-    }
-
-    try {
-      ConfigCommand.main(args)
-    } catch {
-      case e: RuntimeException =>
-    } finally {
-      Exit.resetExitProcedure()
-    }
-
-    assertEquals(Some(1), exitStatus)
-  }
-
   @Test
   def shouldParseArgumentsForClientsEntityType() {
     testArgumentParse("clients")
@@ -143,7 +91,7 @@ class ConfigCommandTest extends ZooKeeperTestHarness with Logging {
   def shouldFailIfUnrecognisedEntityType(): Unit = {
     val createOpts = new ConfigCommandOptions(Array("--zookeeper", zkConnect,
       "--entity-name", "client", "--entity-type", "not-recognised", "--alter", "--add-config", "a=b,c=d"))
-    ConfigCommand.alterConfig(null, createOpts, new DummyAdminZkClient(zkClient))
+    ConfigCommand.alterConfig(null, createOpts, new TestAdminUtils)
   }
 
   @Test
@@ -154,15 +102,14 @@ class ConfigCommandTest extends ZooKeeperTestHarness with Logging {
       "--alter",
       "--add-config", "a=b,c=d"))
 
-    class TestAdminZkClient(zkClient: KafkaZkClient) extends AdminZkClient(zkClient) {
-      override def changeClientIdConfig(clientId: String, configChange: Properties): Unit = {
+    val configChange = new TestAdminUtils {
+      override def changeClientIdConfig(zkUtils: ZkUtils, clientId: String, configChange: Properties): Unit = {
         assertEquals("my-client-id", clientId)
         assertEquals("b", configChange.get("a"))
         assertEquals("d", configChange.get("c"))
       }
     }
-
-    ConfigCommand.alterConfig(null, createOpts, new TestAdminZkClient(zkClient))
+    ConfigCommand.alterConfig(null, createOpts, configChange)
   }
 
   @Test
@@ -173,91 +120,32 @@ class ConfigCommandTest extends ZooKeeperTestHarness with Logging {
       "--alter",
       "--add-config", "a=b,c=d"))
 
-    class TestAdminZkClient(zkClient: KafkaZkClient) extends AdminZkClient(zkClient) {
-      override def changeTopicConfig(topic: String, configChange: Properties): Unit = {
+    val configChange = new TestAdminUtils {
+      override def changeTopicConfig(zkUtils: ZkUtils, topic: String, configChange: Properties): Unit = {
         assertEquals("my-topic", topic)
         assertEquals("b", configChange.get("a"))
         assertEquals("d", configChange.get("c"))
       }
     }
-
-    ConfigCommand.alterConfig(null, createOpts, new TestAdminZkClient(zkClient))
+    ConfigCommand.alterConfig(null, createOpts, configChange)
   }
 
   @Test
-  def shouldAddBrokerQuotaConfig(): Unit = {
-    val alterOpts = new ConfigCommandOptions(Array("--zookeeper", zkConnect,
+  def shouldAddBrokerConfig(): Unit = {
+    val createOpts = new ConfigCommandOptions(Array("--zookeeper", zkConnect,
       "--entity-name", "1",
       "--entity-type", "brokers",
       "--alter",
-      "--add-config", "leader.replication.throttled.rate=10,follower.replication.throttled.rate=20"))
+      "--add-config", "a=b,c=d"))
 
-    class TestAdminZkClient(zkClient: KafkaZkClient) extends AdminZkClient(zkClient) {
-      override def changeBrokerConfig(brokerIds: Seq[Int], configChange: Properties): Unit = {
+    val configChange = new TestAdminUtils {
+      override def changeBrokerConfig(zkUtils: ZkUtils, brokerIds: Seq[Int], configChange: Properties): Unit = {
         assertEquals(Seq(1), brokerIds)
-        assertEquals("10", configChange.get("leader.replication.throttled.rate"))
-        assertEquals("20", configChange.get("follower.replication.throttled.rate"))
+        assertEquals("b", configChange.get("a"))
+        assertEquals("d", configChange.get("c"))
       }
     }
-
-    ConfigCommand.alterConfig(null, alterOpts, new TestAdminZkClient(zkClient))
-  }
-
-  @Test
-  def shouldAddBrokerDynamicConfig(): Unit = {
-    val node = new Node(1, "localhost", 9092)
-    verifyAlterBrokerConfig(node, "1", List("--entity-name", "1"))
-  }
-
-  @Test
-  def shouldAddDefaultBrokerDynamicConfig(): Unit = {
-    val node = new Node(1, "localhost", 9092)
-    verifyAlterBrokerConfig(node, "", List("--entity-default"))
-  }
-
-  def verifyAlterBrokerConfig(node: Node, resourceName: String, resourceOpts: List[String]): Unit = {
-    val optsList = List("--bootstrap-server", "localhost:9092",
-      "--entity-type", "brokers",
-      "--alter",
-      "--add-config", "message.max.bytes=10") ++ resourceOpts
-    val alterOpts = new ConfigCommandOptions(optsList.toArray)
-    val brokerConfigs = mutable.Map[String, String]("num.io.threads" -> "5")
-
-    val resource = new ConfigResource(ConfigResource.Type.BROKER, resourceName)
-    val configEntries = util.Collections.singletonList(new ConfigEntry("num.io.threads", "5"))
-    val future = new KafkaFutureImpl[util.Map[ConfigResource, Config]]
-    future.complete(util.Collections.singletonMap(resource, new Config(configEntries)))
-    val describeResult = EasyMock.createNiceMock(classOf[DescribeConfigsResult])
-    EasyMock.expect(describeResult.all()).andReturn(future).once()
-
-    val alterFuture = new KafkaFutureImpl[Void]
-    alterFuture.complete(null)
-    val alterResult = EasyMock.createNiceMock(classOf[AlterConfigsResult])
-    EasyMock.expect(alterResult.all()).andReturn(alterFuture)
-
-    val mockAdminClient = new MockAdminClient(util.Collections.singletonList(node), node) {
-      override def describeConfigs(resources: util.Collection[ConfigResource], options: DescribeConfigsOptions): DescribeConfigsResult = {
-        assertEquals(1, resources.size)
-        val resource = resources.iterator.next
-        assertEquals(ConfigResource.Type.BROKER, resource.`type`)
-        assertEquals(resourceName, resource.name)
-        describeResult
-      }
-
-      override def alterConfigs(configs: util.Map[ConfigResource, Config], options: AlterConfigsOptions): AlterConfigsResult = {
-        assertEquals(1, configs.size)
-        val entry = configs.entrySet.iterator.next
-        val resource = entry.getKey
-        val config = entry.getValue
-        assertEquals(ConfigResource.Type.BROKER, resource.`type`)
-        config.entries.asScala.foreach { e => brokerConfigs.put(e.name, e.value) }
-        alterResult
-      }
-    }
-    EasyMock.replay(alterResult, describeResult)
-    ConfigCommand.alterBrokerConfig(mockAdminClient, alterOpts, resourceName)
-    assertEquals(Map("message.max.bytes" -> "10", "num.io.threads" -> "5"), brokerConfigs.toMap)
-    EasyMock.reset(alterResult, describeResult)
+    ConfigCommand.alterConfig(null, createOpts, configChange)
   }
 
   @Test
@@ -268,18 +156,15 @@ class ConfigCommandTest extends ZooKeeperTestHarness with Logging {
       "--alter",
       "--add-config", "a=b,c=[d,e ,f],g=[h,i]"))
 
-    class TestAdminZkClient(zkClient: KafkaZkClient) extends AdminZkClient(zkClient) {
-      override def changeBrokerConfig(brokerIds: Seq[Int], configChange: Properties): Unit = {
+    val configChange = new TestAdminUtils {
+      override def changeBrokerConfig(zkUtils: ZkUtils, brokerIds: Seq[Int], configChange: Properties): Unit = {
         assertEquals(Seq(1), brokerIds)
         assertEquals("b", configChange.get("a"))
         assertEquals("d,e ,f", configChange.get("c"))
         assertEquals("h,i", configChange.get("g"))
       }
-
-      override def changeTopicConfig(topic: String, configs: Properties): Unit = {}
     }
-
-    ConfigCommand.alterConfig(null, createOpts, new TestAdminZkClient(zkClient))
+    ConfigCommand.alterConfig(null, createOpts, configChange)
   }
 
   @Test (expected = classOf[IllegalArgumentException])
@@ -288,18 +173,8 @@ class ConfigCommandTest extends ZooKeeperTestHarness with Logging {
       "--entity-name", "1,2,3", //Don't support multiple brokers currently
       "--entity-type", "brokers",
       "--alter",
-      "--add-config", "leader.replication.throttled.rate=10"))
-    ConfigCommand.alterConfig(null, createOpts, new DummyAdminZkClient(zkClient))
-  }
-
-  @Test (expected = classOf[IllegalArgumentException])
-  def shouldNotUpdateDynamicBrokerConfigUsingZooKeeper(): Unit = {
-    val createOpts = new ConfigCommandOptions(Array("--zookeeper", zkConnect,
-      "--entity-name", "1",
-      "--entity-type", "brokers",
-      "--alter",
-      "--add-config", "message.max.size=100000"))
-    ConfigCommand.alterConfig(null, createOpts, new DummyAdminZkClient(zkClient))
+      "--add-config", "a=b"))
+    ConfigCommand.alterConfig(null, createOpts, new TestAdminUtils)
   }
 
   @Test (expected = classOf[IllegalArgumentException])
@@ -309,7 +184,7 @@ class ConfigCommandTest extends ZooKeeperTestHarness with Logging {
       "--entity-type", "brokers",
       "--alter",
       "--add-config", "a="))
-    ConfigCommand.alterConfig(null, createOpts, new DummyAdminZkClient(zkClient))
+    ConfigCommand.alterConfig(null, createOpts, new TestAdminUtils)
   }
 
   @Test (expected = classOf[IllegalArgumentException])
@@ -319,17 +194,7 @@ class ConfigCommandTest extends ZooKeeperTestHarness with Logging {
       "--entity-type", "brokers",
       "--alter",
       "--add-config", "a=[b,c,d=e"))
-    ConfigCommand.alterConfig(null, createOpts, new DummyAdminZkClient(zkClient))
-  }
-
-  @Test (expected = classOf[InvalidConfigException])
-  def shouldNotUpdateBrokerConfigIfNonExistingConfigIsDeleted(): Unit = {
-    val createOpts = new ConfigCommandOptions(Array("--zookeeper", zkConnect,
-      "--entity-name", "my-topic",
-      "--entity-type", "topics",
-      "--alter",
-      "--delete-config", "missing_config1, missing_config2"))
-    ConfigCommand.alterConfig(null, createOpts, new DummyAdminZkClient(zkClient))
+    ConfigCommand.alterConfig(null, createOpts, new TestAdminUtils)
   }
 
   @Test
@@ -340,8 +205,8 @@ class ConfigCommandTest extends ZooKeeperTestHarness with Logging {
       "--alter",
       "--delete-config", "a,c"))
 
-    class TestAdminZkClient(zkClient: KafkaZkClient) extends AdminZkClient(zkClient) {
-      override def fetchEntityConfig(entityType: String, entityName: String): Properties = {
+    val configChange = new TestAdminUtils {
+      override def fetchEntityConfig(zkUtils: ZkUtils, entityType: String, entityName: String): Properties = {
         val properties: Properties = new Properties
         properties.put("a", "b")
         properties.put("c", "d")
@@ -349,58 +214,12 @@ class ConfigCommandTest extends ZooKeeperTestHarness with Logging {
         properties
       }
 
-      override def changeBrokerConfig(brokerIds: Seq[Int], configChange: Properties): Unit = {
+      override def changeBrokerConfig(zkUtils: ZkUtils, brokerIds: Seq[Int], configChange: Properties): Unit = {
         assertEquals("f", configChange.get("e"))
         assertEquals(1, configChange.size())
       }
     }
-
-    ConfigCommand.alterConfig(null, createOpts, new TestAdminZkClient(zkClient))
-  }
-
-  @Test
-  def testScramCredentials(): Unit = {
-    def createOpts(user: String, config: String): ConfigCommandOptions = {
-      new ConfigCommandOptions(Array("--zookeeper", zkConnect,
-        "--entity-name", user,
-        "--entity-type", "users",
-        "--alter",
-        "--add-config", config))
-    }
-
-    def deleteOpts(user: String, mechanism: String) = new ConfigCommandOptions(Array("--zookeeper", zkConnect,
-        "--entity-name", user,
-        "--entity-type", "users",
-        "--alter",
-        "--delete-config", mechanism))
-
-    val credentials = mutable.Map[String, Properties]()
-    case class CredentialChange(user: String, mechanisms: Set[String], iterations: Int) extends AdminZkClient(zkClient) {
-      override def fetchEntityConfig(entityType: String, entityName: String): Properties = {
-        credentials.getOrElse(entityName, new Properties())
-      }
-      override def changeUserOrUserClientIdConfig(sanitizedEntityName: String, configChange: Properties): Unit = {
-        assertEquals(user, sanitizedEntityName)
-        assertEquals(mechanisms, configChange.keySet().asScala)
-        for (mechanism <- mechanisms) {
-          val value = configChange.getProperty(mechanism)
-          assertEquals(-1, value.indexOf("password="))
-          val scramCredential = ScramCredentialUtils.credentialFromString(value)
-          assertEquals(iterations, scramCredential.iterations)
-          if (configChange != null)
-              credentials.put(user, configChange)
-        }
-      }
-    }
-    val optsA = createOpts("userA", "SCRAM-SHA-256=[iterations=8192,password=abc, def]")
-    ConfigCommand.alterConfig(null, optsA, CredentialChange("userA", Set("SCRAM-SHA-256"), 8192))
-    val optsB = createOpts("userB", "SCRAM-SHA-256=[iterations=4096,password=abc, def],SCRAM-SHA-512=[password=1234=abc]")
-    ConfigCommand.alterConfig(null, optsB, CredentialChange("userB", Set("SCRAM-SHA-256", "SCRAM-SHA-512"), 4096))
-
-    val del256 = deleteOpts("userB", "SCRAM-SHA-256")
-    ConfigCommand.alterConfig(null, del256, CredentialChange("userB", Set("SCRAM-SHA-512"), 4096))
-    val del512 = deleteOpts("userB", "SCRAM-SHA-512")
-    ConfigCommand.alterConfig(null, del512, CredentialChange("userB", Set(), 4096))
+    ConfigCommand.alterConfig(null, createOpts, configChange)
   }
 
   @Test
@@ -431,7 +250,7 @@ class ConfigCommandTest extends ZooKeeperTestHarness with Logging {
         ConfigCommand.parseEntity(opts)
         fail("Did not fail with invalid argument list")
       } catch {
-        case _: IllegalArgumentException => // expected exception
+        case e: IllegalArgumentException => // expected exception
       }
     }
 
@@ -449,9 +268,9 @@ class ConfigCommandTest extends ZooKeeperTestHarness with Logging {
 
     // <user> quota
     val principal = "CN=ConfigCommandTest,O=Apache,L=<default>"
-    val sanitizedPrincipal = Sanitizer.sanitize(principal)
+    val sanitizedPrincipal = QuotaId.sanitize(principal)
     assertEquals(-1, sanitizedPrincipal.indexOf('='))
-    assertEquals(principal, Sanitizer.desanitize(sanitizedPrincipal))
+    assertEquals(principal, QuotaId.desanitize(sanitizedPrincipal))
     for (opts <- Seq(describeOpts, alterOpts)) {
       checkEntity("users", Some(principal), sanitizedPrincipal, opts)
       checkEntity("users", Some(""), ConfigEntityName.Default, opts)
@@ -485,56 +304,58 @@ class ConfigCommandTest extends ZooKeeperTestHarness with Logging {
       assertEquals(expectedEntityName, entity.fullSanitizedName)
     }
 
-    // <default> is a valid user principal and client-id (can be handled with URL-encoding),
-    checkEntity("users", Sanitizer.sanitize("<default>"),
+    // <default> is a valid user principal (can be handled with URL-encoding),
+    // but an invalid client-id (cannot be handled since client-ids are not encoded)
+    checkEntity("users", QuotaId.sanitize("<default>"),
         "--entity-type", "users", "--entity-name", "<default>",
         "--alter", "--add-config", "a=b,c=d")
-    checkEntity("clients", Sanitizer.sanitize("<default>"),
-        "--entity-type", "clients", "--entity-name", "<default>",
-        "--alter", "--add-config", "a=b,c=d")
+    try {
+      checkEntity("clients", QuotaId.sanitize("<default>"),
+          "--entity-type", "clients", "--entity-name", "<default>",
+          "--alter", "--add-config", "a=b,c=d")
+      fail("Did not fail with invalid client-id")
+    } catch {
+      case e: InvalidConfigException => // expected
+    }
 
-
-    checkEntity("users", Sanitizer.sanitize("CN=user1") + "/clients/client1",
+    checkEntity("users", QuotaId.sanitize("CN=user1") + "/clients/client1",
         "--entity-type", "users", "--entity-name", "CN=user1", "--entity-type", "clients", "--entity-name", "client1",
         "--alter", "--add-config", "a=b,c=d")
-    checkEntity("users", Sanitizer.sanitize("CN=user1") + "/clients/client1",
+    checkEntity("users", QuotaId.sanitize("CN=user1") + "/clients/client1",
         "--entity-name", "CN=user1", "--entity-type", "users", "--entity-name", "client1", "--entity-type", "clients",
         "--alter", "--add-config", "a=b,c=d")
-    checkEntity("users", Sanitizer.sanitize("CN=user1") + "/clients/client1",
+    checkEntity("users", QuotaId.sanitize("CN=user1") + "/clients/client1",
         "--entity-type", "clients", "--entity-name", "client1", "--entity-type", "users", "--entity-name", "CN=user1",
         "--alter", "--add-config", "a=b,c=d")
-    checkEntity("users", Sanitizer.sanitize("CN=user1") + "/clients/client1",
+    checkEntity("users", QuotaId.sanitize("CN=user1") + "/clients/client1",
         "--entity-name", "client1", "--entity-type", "clients", "--entity-name", "CN=user1", "--entity-type", "users",
         "--alter", "--add-config", "a=b,c=d")
-    checkEntity("users", Sanitizer.sanitize("CN=user1") + "/clients",
+    checkEntity("users", QuotaId.sanitize("CN=user1") + "/clients",
         "--entity-type", "clients", "--entity-name", "CN=user1", "--entity-type", "users",
         "--describe")
     checkEntity("users", "/clients",
         "--entity-type", "clients", "--entity-type", "users",
         "--describe")
-    checkEntity("users", Sanitizer.sanitize("CN=user1") + "/clients/" + Sanitizer.sanitize("client1?@%"),
-        "--entity-name", "client1?@%", "--entity-type", "clients", "--entity-name", "CN=user1", "--entity-type", "users",
-        "--alter", "--add-config", "a=b,c=d")
   }
 
   @Test
   def testQuotaDescribeEntities() {
-    val zkClient = EasyMock.createNiceMock(classOf[KafkaZkClient])
+    val zkUtils = EasyMock.createNiceMock(classOf[ZkUtils])
 
     def checkEntities(opts: Array[String], expectedFetches: Map[String, Seq[String]], expectedEntityNames: Seq[String]) {
       val entity = ConfigCommand.parseEntity(new ConfigCommandOptions(opts :+ "--describe"))
       expectedFetches.foreach {
-        case (name, values) => EasyMock.expect(zkClient.getAllEntitiesWithConfig(name)).andReturn(values)
+        case (name, values) => EasyMock.expect(zkUtils.getAllEntitiesWithConfig(name)).andReturn(values)
       }
-      EasyMock.replay(zkClient)
-      val entities = entity.getAllEntities(zkClient)
+      EasyMock.replay(zkUtils)
+      val entities = entity.getAllEntities(zkUtils)
       assertEquals(expectedEntityNames, entities.map(e => e.fullSanitizedName))
-      EasyMock.reset(zkClient)
+      EasyMock.reset(zkUtils)
     }
 
     val clientId = "a-client"
     val principal = "CN=ConfigCommandTest.testQuotaDescribeEntities , O=Apache, L=<default>"
-    val sanitizedPrincipal = Sanitizer.sanitize(principal)
+    val sanitizedPrincipal = QuotaId.sanitize(principal)
     val userClient = sanitizedPrincipal + "/clients/" + clientId
 
     var opts = Array("--entity-type", "clients", "--entity-name", clientId)
@@ -578,13 +399,4 @@ class ConfigCommandTest extends ZooKeeperTestHarness with Logging {
         Map("users" -> Seq("<default>", sanitizedPrincipal)) ++ defaultUserMap ++ userMap,
         Seq("<default>/clients/client-3", sanitizedPrincipal + "/clients/client-2"))
   }
-
-  class DummyAdminZkClient(zkClient: KafkaZkClient) extends AdminZkClient(zkClient) {
-    override def changeBrokerConfig(brokerIds: Seq[Int], configs: Properties): Unit = {}
-    override def fetchEntityConfig(entityType: String, entityName: String): Properties = {new Properties}
-    override def changeClientIdConfig(clientId: String, configs: Properties): Unit = {}
-    override def changeUserOrUserClientIdConfig(sanitizedEntityName: String, configs: Properties): Unit = {}
-    override def changeTopicConfig(topic: String, configs: Properties): Unit = {}
-  }
-
 }
