@@ -12,23 +12,9 @@
   */
 package kafka.api
 
-import kafka.utils.JaasTestUtils
-import org.apache.kafka.common.security.auth.SecurityProtocol
-import org.junit.{After, Before}
+import org.apache.kafka.common.protocol.SecurityProtocol
 
-class SaslPlaintextConsumerTest extends BaseConsumerTest with SaslSetup {
+class SaslPlaintextConsumerTest extends BaseConsumerTest with SaslTestHarness {
+  override protected val zkSaslEnabled = false
   override protected def securityProtocol = SecurityProtocol.SASL_PLAINTEXT
-
-  @Before
-  override def setUp(): Unit = {
-    startSasl(jaasSections(Seq("GSSAPI"), Some("GSSAPI"), KafkaSasl, JaasTestUtils.KafkaServerContextName))
-    super.setUp()
-  }
-
-  @After
-  override def tearDown(): Unit = {
-    super.tearDown()
-    closeSasl()
-  }
-
 }
