@@ -14,26 +14,13 @@ package kafka.api
 
 import java.io.File
 
+import org.apache.kafka.common.protocol.SecurityProtocol
 import kafka.server.KafkaConfig
-import kafka.utils.JaasTestUtils
-import org.apache.kafka.common.security.auth.SecurityProtocol
-import org.junit.{After, Before}
 
-class SaslSslConsumerTest extends BaseConsumerTest with SaslSetup {
+class SaslSslConsumerTest extends BaseConsumerTest with SaslTestHarness {
+  override protected val zkSaslEnabled = true
   this.serverConfig.setProperty(KafkaConfig.ZkEnableSecureAclsProp, "true")
   override protected def securityProtocol = SecurityProtocol.SASL_SSL
   override protected lazy val trustStoreFile = Some(File.createTempFile("truststore", ".jks"))
-
-  @Before
-  override def setUp(): Unit = {
-    startSasl(jaasSections(Seq("GSSAPI"), Some("GSSAPI"), Both, JaasTestUtils.KafkaServerContextName))
-    super.setUp()
-  }
-
-  @After
-  override def tearDown(): Unit = {
-    super.tearDown()
-    closeSasl()
-  }
   
 }
